/* automatically generated from weather-config.ui */
#ifdef __SUNPRO_C
#pragma align 4 (weather_config_ui)
#endif
#ifdef __GNUC__
static const char weather_config_ui[] __attribute__ ((__aligned__ (4))) =
#else
static const char weather_config_ui[] =
#endif
{
  "<?xml version=\"1.0\" encoding=\"UTF-8\"?><interface><requires lib=\"gt"
  "k+\" version=\"3.20\"/><requires lib=\"libxfce4ui-2\" version=\"4.12\"/"
  "><object class=\"GtkAdjustment\" id=\"adjustment1\"><property name=\"lo"
  "wer\">1</property><property name=\"upper\">10</property><property name="
  "\"step_increment\">1</property><property name=\"page_increment\">10</pr"
  "operty></object><object class=\"GtkAdjustment\" id=\"adjustment2\"><pro"
  "perty name=\"lower\">1</property><property name=\"upper\">10</property>"
  "<property name=\"step_increment\">1</property><property name=\"page_inc"
  "rement\">10</property></object><object class=\"GtkAdjustment\" id=\"adj"
  "ustment3\"><property name=\"lower\">-90</property><property name=\"uppe"
  "r\">90</property><property name=\"step_increment\">1</property><propert"
  "y name=\"page_increment\">10</property></object><object class=\"GtkAdju"
  "stment\" id=\"adjustment4\"><property name=\"lower\">-180</property><pr"
  "operty name=\"upper\">180</property><property name=\"step_increment\">1"
  "</property><property name=\"page_increment\">10</property></object><obj"
  "ect class=\"GtkAdjustment\" id=\"adjustment5\"><property name=\"lower\""
  ">-420</property><property name=\"upper\">10000</property><property name"
  "=\"step_increment\">1</property><property name=\"page_increment\">10</p"
  "roperty></object><object class=\"GtkImage\" id=\"image1\"><property nam"
  "e=\"visible\">True</property><property name=\"can_focus\">False</proper"
  "ty><property name=\"icon_name\">edit-find</property></object><object cl"
  "ass=\"GtkImage\" id=\"image2\"><property name=\"visible\">True</propert"
  "y><property name=\"can_focus\">False</property><property name=\"icon_na"
  "me\">folder</property></object><object class=\"GtkImage\" id=\"image3\""
  "><property name=\"visible\">True</property><property name=\"can_focus\""
  ">False</property><property name=\"icon_name\">list-add-symbolic</proper"
  "ty></object><object class=\"GtkImage\" id=\"image4\"><property name=\"v"
  "isible\">True</property><property name=\"can_focus\">False</property><p"
  "roperty name=\"icon_name\">list-remove-symbolic</property></object><obj"
  "ect class=\"GtkImage\" id=\"image5\"><property name=\"visible\">True</p"
  "roperty><property name=\"can_focus\">False</property><property name=\"i"
  "con_name\">go-up-symbolic</property></object><object class=\"GtkImage\""
  " id=\"image6\"><property name=\"visible\">True</property><property name"
  "=\"can_focus\">False</property><property name=\"icon_name\">go-down-sym"
  "bolic</property></object><object class=\"GtkImage\" id=\"image7\"><prop"
  "erty name=\"visible\">True</property><property name=\"can_focus\">False"
  "</property><property name=\"icon_name\">window-close-symbolic</property"
  "></object><object class=\"GtkImage\" id=\"image8\"><property name=\"vis"
  "ible\">True</property><property name=\"can_focus\">False</property><pro"
  "perty name=\"icon_name\">help-browser</property></object><object class="
  "\"GtkListStore\" id=\"liststore1\"/><object class=\"XfceTitledDialog\" "
  "id=\"dialog\"><property name=\"can_focus\">False</property><property na"
  "me=\"title\" translatable=\"yes\">Weather Update</property><property na"
  "me=\"default_width\">430</property><property name=\"default_height\">43"
  "0</property><property name=\"destroy_with_parent\">True</property><prop"
  "erty name=\"icon_name\">org.xfce.panel.weather</property><property name"
  "=\"type_hint\">dialog</property><child internal-child=\"vbox\"><object "
  "class=\"GtkBox\" id=\"box1\"><property name=\"can_focus\">False</proper"
  "ty><property name=\"orientation\">vertical</property><property name=\"s"
  "pacing\">2</property><child internal-child=\"action_area\"><object clas"
  "s=\"GtkButtonBox\" id=\"buttonbox1\"><property name=\"can_focus\">False"
  "</property><property name=\"layout_style\">end</property><child><object"
  " class=\"GtkButton\" id=\"help_button\"><property name=\"label\" transl"
  "atable=\"yes\">_Help</property><property name=\"visible\">True</propert"
  "y><property name=\"can_focus\">True</property><property name=\"receives"
  "_default\">True</property><property name=\"image\">image8</property><pr"
  "operty name=\"use_underline\">True</property></object><packing><propert"
  "y name=\"expand\">True</property><property name=\"fill\">True</property"
  "><property name=\"position\">0</property><property name=\"secondary\">T"
  "rue</property></packing></child><child><object class=\"GtkButton\" id=\""
  "close_button\"><property name=\"label\" translatable=\"yes\">_Close</pr"
  "operty><property name=\"visible\">True</property><property name=\"can_f"
  "ocus\">True</property><property name=\"receives_default\">True</propert"
  "y><property name=\"image\">image7</property><property name=\"use_underl"
  "ine\">True</property></object><packing><property name=\"expand\">True</"
  "property><property name=\"fill\">True</property><property name=\"positi"
  "on\">1</property></packing></child></object><packing><property name=\"e"
  "xpand\">False</property><property name=\"fill\">False</property><proper"
  "ty name=\"position\">0</property></packing></child><child><object class"
  "=\"GtkNotebook\" id=\"notebook\"><property name=\"visible\">True</prope"
  "rty><property name=\"can_focus\">True</property><property name=\"border"
  "_width\">6</property><child><object class=\"GtkGrid\" id=\"grid1\"><pro"
  "perty name=\"visible\">True</property><property name=\"can_focus\">Fals"
  "e</property><property name=\"valign\">start</property><property name=\""
  "border_width\">12</property><property name=\"row_spacing\">6</property>"
  "<property name=\"column_spacing\">12</property><child><object class=\"G"
  "tkLabel\" id=\"label1\"><property name=\"visible\">True</property><prop"
  "erty name=\"can_focus\">False</property><property name=\"label\" transl"
  "atable=\"yes\">Location _name:</property><property name=\"use_underline"
  "\">True</property><property name=\"mnemonic_widget\">text_loc_name</pro"
  "perty><property name=\"xalign\">0</property></object><packing><property"
  " name=\"left_attach\">0</property><property name=\"top_attach\">1</prop"
  "erty></packing></child><child><object class=\"GtkLabel\" id=\"label2\">"
  "<property name=\"visible\">True</property><property name=\"can_focus\">"
  "False</property><property name=\"label\" translatable=\"yes\">Latitud_e"
  ":</property><property name=\"use_underline\">True</property><property n"
  "ame=\"mnemonic_widget\">spin_lat</property><property name=\"xalign\">0<"
  "/property></object><packing><property name=\"left_attach\">0</property>"
  "<property name=\"top_attach\">2</property></packing></child><child><obj"
  "ect class=\"GtkLabel\" id=\"label3\"><property name=\"visible\">True</p"
  "roperty><property name=\"can_focus\">False</property><property name=\"l"
  "abel\" translatable=\"yes\">L_ongitude:</property><property name=\"use_"
  "underline\">True</property><property name=\"mnemonic_widget\">spin_lon<"
  "/property><property name=\"xalign\">0</property></object><packing><prop"
  "erty name=\"left_attach\">0</property><property name=\"top_attach\">3</"
  "property></packing></child><child><object class=\"GtkLabel\" id=\"label"
  "4\"><property name=\"visible\">True</property><property name=\"can_focu"
  "s\">False</property><property name=\"label\" translatable=\"yes\">Altit"
  "u_de:</property><property name=\"use_underline\">True</property><proper"
  "ty name=\"mnemonic_widget\">spin_alt</property><property name=\"xalign\""
  ">0</property></object><packing><property name=\"left_attach\">0</proper"
  "ty><property name=\"top_attach\">4</property></packing></child><child><"
  "object class=\"GtkLabel\" id=\"label5\"><property name=\"visible\">True"
  "</property><property name=\"can_focus\">False</property><property name="
  "\"label\" translatable=\"yes\">_Timezone:</property><property name=\"us"
  "e_underline\">True</property><property name=\"mnemonic_widget\">text_ti"
  "mezone</property><property name=\"xalign\">0</property></object><packin"
  "g><property name=\"left_attach\">0</property><property name=\"top_attac"
  "h\">5</property></packing></child><child><object class=\"GtkBox\" id=\""
  "box2\"><property name=\"visible\">True</property><property name=\"can_f"
  "ocus\">False</property><property name=\"spacing\">6</property><child><o"
  "bject class=\"GtkSpinButton\" id=\"spin_lat\"><property name=\"visible\""
  ">True</property><property name=\"can_focus\">True</property><property n"
  "ame=\"tooltip_text\" translatable=\"yes\">Latitude specifies the north-"
  "south position of a point on the Earth\'s surface. If you change this v"
  "alue manually, you need to provide altitude and timezone manually too.<"
  "/property><property name=\"width_chars\">11</property><property name=\""
  "adjustment\">adjustment3</property><property name=\"digits\">6</propert"
  "y></object><packing><property name=\"expand\">False</property><property"
  " name=\"fill\">True</property><property name=\"position\">0</property><"
  "/packing></child><child><object class=\"GtkLabel\" id=\"label7\"><prope"
  "rty name=\"visible\">True</property><property name=\"can_focus\">False<"
  "/property><property name=\"label\" translatable=\"yes\">\302\260</prope"
  "rty></object><packing><property name=\"expand\">False</property><proper"
  "ty name=\"fill\">True</property><property name=\"position\">1</property"
  "></packing></child></object><packing><property name=\"left_attach\">1</"
  "property><property name=\"top_attach\">2</property></packing></child><c"
  "hild><object class=\"GtkBox\" id=\"box3\"><property name=\"visible\">Tr"
  "ue</property><property name=\"can_focus\">False</property><property nam"
  "e=\"spacing\">6</property><child><object class=\"GtkSpinButton\" id=\"s"
  "pin_lon\"><property name=\"visible\">True</property><property name=\"ca"
  "n_focus\">True</property><property name=\"tooltip_text\" translatable=\""
  "yes\">Longitude specifies the east-west position of a point on the Eart"
  "h\'s surface. If you change this value manually, you need to provide al"
  "titude and timezone manually too.</property><property name=\"width_char"
  "s\">11</property><property name=\"adjustment\">adjustment4</property><p"
  "roperty name=\"digits\">6</property></object><packing><property name=\""
  "expand\">False</property><property name=\"fill\">True</property><proper"
  "ty name=\"position\">0</property></packing></child><child><object class"
  "=\"GtkLabel\" id=\"label8\"><property name=\"visible\">True</property><"
  "property name=\"can_focus\">False</property><property name=\"label\" tr"
  "anslatable=\"yes\">\302\260</property></object><packing><property name="
  "\"expand\">False</property><property name=\"fill\">True</property><prop"
  "erty name=\"position\">1</property></packing></child></object><packing>"
  "<property name=\"left_attach\">1</property><property name=\"top_attach\""
  ">3</property></packing></child><child><object class=\"GtkBox\" id=\"box"
  "4\"><property name=\"visible\">True</property><property name=\"can_focu"
  "s\">False</property><property name=\"spacing\">6</property><child><obje"
  "ct class=\"GtkSpinButton\" id=\"spin_alt\"><property name=\"visible\">T"
  "rue</property><property name=\"can_focus\">True</property><property nam"
  "e=\"tooltip_text\" translatable=\"yes\">For locations outside Norway th"
  "e elevation model that\'s used by the met.no webservice is not very goo"
  "d, so it\'s usually necessary to specify the altitude as an additional "
  "parameter, otherwise the reported values will not be correct.\n\nThe pl"
  "ugin tries to auto-detect the altitude using the GeoNames webservice, b"
  "ut that might not always be correct too, so you can change it here.\n\n"
  "Altitude is given in meters above sea level, or alternatively in feet b"
  "y changing the unit on the units page. It should match the real value r"
  "oughly, but small differences will have no influence on the weather dat"
  "a. Inside Norway, this setting has no effect at all.</property><propert"
  "y name=\"width_chars\">11</property><property name=\"adjustment\">adjus"
  "tment5</property></object><packing><property name=\"expand\">False</pro"
  "perty><property name=\"fill\">True</property><property name=\"position\""
  ">0</property></packing></child><child><object class=\"GtkLabel\" id=\"l"
  "abel_alt_unit\"><property name=\"visible\">True</property><property nam"
  "e=\"can_focus\">False</property><property name=\"label\" translatable=\""
  "yes\">feet</property></object><packing><property name=\"expand\">False<"
  "/property><property name=\"fill\">True</property><property name=\"posit"
  "ion\">1</property></packing></child></object><packing><property name=\""
  "left_attach\">1</property><property name=\"top_attach\">4</property></p"
  "acking></child><child><object class=\"GtkEntry\" id=\"text_timezone\"><"
  "property name=\"visible\">True</property><property name=\"can_focus\">T"
  "rue</property><property name=\"tooltip_markup\" translatable=\"yes\">If"
  " the chosen location is not in your current timezone, then it is necess"
  "ary to &lt;i&gt;put&lt;/i&gt; the plugin into that other timezone for t"
  "he times to be shown correctly. The proper timezone will be auto-detect"
  "ed via the GeoNames web service, but you might want to correct it if ne"
  "cessary.\nLeave this field empty to use the timezone set by your system"
  ". Invalid entries will cause the use of UTC time, but that may also dep"
  "end on your system.</property></object><packing><property name=\"left_a"
  "ttach\">1</property><property name=\"top_attach\">5</property></packing"
  "></child><child><object class=\"GtkBox\"><property name=\"visible\">Tru"
  "e</property><property name=\"can_focus\">False</property><property name"
  "=\"hexpand\">True</property><property name=\"spacing\">6</property><chi"
  "ld><object class=\"GtkEntry\" id=\"text_loc_name\"><property name=\"vis"
  "ible\">True</property><property name=\"can_focus\">True</property><prop"
  "erty name=\"tooltip_text\" translatable=\"yes\">Change the name for the"
  " location to your liking. It is used for display and does not affect th"
  "e location parameters in any way.</property><property name=\"hexpand\">"
  "True</property></object><packing><property name=\"expand\">False</prope"
  "rty><property name=\"fill\">True</property><property name=\"position\">"
  "0</property></packing></child><child><object class=\"GtkButton\" id=\"b"
  "utton_loc_change\"><property name=\"label\" translatable=\"yes\">Change"
  "...</property><property name=\"visible\">True</property><property name="
  "\"can_focus\">True</property><property name=\"receives_default\">True</"
  "property><property name=\"tooltip_text\" translatable=\"yes\">Search fo"
  "r a new location and auto-detect its parameters.</property><property na"
  "me=\"image\">image1</property></object><packing><property name=\"expand"
  "\">False</property><property name=\"fill\">True</property><property nam"
  "e=\"position\">1</property></packing></child><child><object class=\"Gtk"
  "Spinner\" id=\"update_spinner\"><property name=\"can_focus\">False</pro"
  "perty><property name=\"no_show_all\">True</property></object><packing><"
  "property name=\"expand\">False</property><property name=\"fill\">True</"
  "property><property name=\"position\">2</property></packing></child></ob"
  "ject><packing><property name=\"left_attach\">1</property><property name"
  "=\"top_attach\">1</property></packing></child><child><object class=\"Gt"
  "kInfoBar\"><property name=\"visible\">True</property><property name=\"c"
  "an_focus\">False</property><child internal-child=\"action_area\"><objec"
  "t class=\"GtkButtonBox\"><property name=\"can_focus\">False</property><"
  "property name=\"layout_style\">end</property><child><placeholder/></chi"
  "ld></object><packing><property name=\"expand\">False</property><propert"
  "y name=\"fill\">False</property><property name=\"position\">0</property"
  "></packing></child><child internal-child=\"content_area\"><object class"
  "=\"GtkBox\"><property name=\"can_focus\">False</property><property name"
  "=\"spacing\">16</property><child><object class=\"GtkImage\"><property n"
  "ame=\"visible\">True</property><property name=\"can_focus\">False</prop"
  "erty><property name=\"icon_name\">dialog-information-symbolic</property"
  "></object><packing><property name=\"expand\">False</property><property "
  "name=\"fill\">True</property><property name=\"position\">0</property></"
  "packing></child><child><object class=\"GtkLabel\" id=\"label6\"><proper"
  "ty name=\"visible\">True</property><property name=\"can_focus\">False</"
  "property><property name=\"halign\">center</property><property name=\"la"
  "bel\" translatable=\"yes\">Please change location name to your liking a"
  "nd correct alltitude and timezone if they are not auto-detected correct"
  "ly.</property><property name=\"wrap\">True</property><property name=\"x"
  "align\">0</property></object><packing><property name=\"expand\">True</p"
  "roperty><property name=\"fill\">True</property><property name=\"positio"
  "n\">1</property></packing></child></object><packing><property name=\"ex"
  "pand\">True</property><property name=\"fill\">True</property><property "
  "name=\"position\">0</property></packing></child></object><packing><prop"
  "erty name=\"left_attach\">0</property><property name=\"top_attach\">0</"
  "property><property name=\"width\">2</property></packing></child></objec"
  "t></child><child type=\"tab\"><object class=\"GtkLabel\" id=\"label9\">"
  "<property name=\"visible\">True</property><property name=\"can_focus\">"
  "False</property><property name=\"label\" translatable=\"yes\">_Location"
  "</property><property name=\"use_underline\">True</property></object><pa"
  "cking><property name=\"tab_fill\">False</property></packing></child><ch"
  "ild><object class=\"GtkGrid\" id=\"grid2\"><property name=\"visible\">T"
  "rue</property><property name=\"can_focus\">False</property><property na"
  "me=\"valign\">start</property><property name=\"border_width\">12</prope"
  "rty><property name=\"row_spacing\">6</property><property name=\"column_"
  "spacing\">12</property><child><object class=\"GtkLabel\" id=\"label10\""
  "><property name=\"visible\">True</property><property name=\"can_focus\""
  ">False</property><property name=\"label\" translatable=\"yes\">_Tempera"
  "ture:</property><property name=\"use_underline\">True</property><proper"
  "ty name=\"mnemonic_widget\">combo_unit_temperature</property><property "
  "name=\"xalign\">0</property></object><packing><property name=\"left_att"
  "ach\">0</property><property name=\"top_attach\">0</property></packing><"
  "/child><child><object class=\"GtkLabel\" id=\"label11\"><property name="
  "\"visible\">True</property><property name=\"can_focus\">False</property"
  "><property name=\"label\" translatable=\"yes\">Barometric _pressure:</p"
  "roperty><property name=\"use_underline\">True</property><property name="
  "\"mnemonic_widget\">combo_unit_pressure</property><property name=\"xali"
  "gn\">0</property></object><packing><property name=\"left_attach\">0</pr"
  "operty><property name=\"top_attach\">1</property></packing></child><chi"
  "ld><object class=\"GtkLabel\" id=\"label12\"><property name=\"visible\""
  ">True</property><property name=\"can_focus\">False</property><property "
  "name=\"label\" translatable=\"yes\">_Wind speed:</property><property na"
  "me=\"use_underline\">True</property><property name=\"mnemonic_widget\">"
  "combo_unit_windspeed</property><property name=\"xalign\">0</property></"
  "object><packing><property name=\"left_attach\">0</property><property na"
  "me=\"top_attach\">2</property></packing></child><child><object class=\""
  "GtkLabel\" id=\"label13\"><property name=\"visible\">True</property><pr"
  "operty name=\"can_focus\">False</property><property name=\"label\" tran"
  "slatable=\"yes\">Prec_ipitations:</property><property name=\"use_underl"
  "ine\">True</property><property name=\"mnemonic_widget\">combo_unit_prec"
  "ipitation</property><property name=\"xalign\">0</property></object><pac"
  "king><property name=\"left_attach\">0</property><property name=\"top_at"
  "tach\">3</property></packing></child><child><object class=\"GtkLabel\" "
  "id=\"label14\"><property name=\"visible\">True</property><property name"
  "=\"can_focus\">False</property><property name=\"label\" translatable=\""
  "yes\">Altitu_de:</property><property name=\"use_underline\">True</prope"
  "rty><property name=\"mnemonic_widget\">combo_unit_altitude</property><p"
  "roperty name=\"xalign\">0</property></object><packing><property name=\""
  "left_attach\">0</property><property name=\"top_attach\">4</property></p"
  "acking></child><child><object class=\"GtkSeparator\" id=\"separator2\">"
  "<property name=\"visible\">True</property><property name=\"can_focus\">"
  "False</property><property name=\"margin_top\">6</property><property nam"
  "e=\"margin_bottom\">6</property></object><packing><property name=\"left"
  "_attach\">0</property><property name=\"top_attach\">5</property><proper"
  "ty name=\"width\">2</property></packing></child><child><object class=\""
  "GtkLabel\" id=\"label15\"><property name=\"visible\">True</property><pr"
  "operty name=\"can_focus\">False</property><property name=\"label\" tran"
  "slatable=\"yes\">Apparent te_mperature:</property><property name=\"use_"
  "underline\">True</property><property name=\"mnemonic_widget\">combo_app"
  "arent_temperature</property></object><packing><property name=\"left_att"
  "ach\">0</property><property name=\"top_attach\">6</property></packing><"
  "/child><child><object class=\"GtkComboBoxText\" id=\"combo_unit_tempera"
  "ture\"><property name=\"visible\">True</property><property name=\"can_f"
  "ocus\">False</property><property name=\"hexpand\">True</property><items"
  "><item translatable=\"yes\">Celsius (\302\260C)</item><item translatabl"
  "e=\"yes\">Fahrenheit (\302\260F)</item></items></object><packing><prope"
  "rty name=\"left_attach\">1</property><property name=\"top_attach\">0</p"
  "roperty></packing></child><child><object class=\"GtkComboBoxText\" id=\""
  "combo_unit_pressure\"><property name=\"visible\">True</property><proper"
  "ty name=\"can_focus\">False</property><items><item translatable=\"yes\""
  ">Hectopascals (hPa)</item><item translatable=\"yes\">Inches of mercury "
  "(inHg)</item><item translatable=\"yes\">Pound-force per square inch (ps"
  "i)</item><item translatable=\"yes\">Torr (mmHg)</item></items></object>"
  "<packing><property name=\"left_attach\">1</property><property name=\"to"
  "p_attach\">1</property></packing></child><child><object class=\"GtkComb"
  "oBoxText\" id=\"combo_unit_windspeed\"><property name=\"visible\">True<"
  "/property><property name=\"can_focus\">False</property><items><item tra"
  "nslatable=\"yes\">Kilometers per hour (km/h)</item><item translatable=\""
  "yes\">Miles per hour (mph)</item><item translatable=\"yes\">Meters per "
  "second (m/s)</item><item translatable=\"yes\">Feet per second (ft/s)</i"
  "tem><item translatable=\"yes\">Knots (kt)</item></items></object><packi"
  "ng><property name=\"left_attach\">1</property><property name=\"top_atta"
  "ch\">2</property></packing></child><child><object class=\"GtkComboBoxTe"
  "xt\" id=\"combo_unit_precipitation\"><property name=\"visible\">True</p"
  "roperty><property name=\"can_focus\">False</property><items><item trans"
  "latable=\"yes\">Millimeters (mm)</item><item translatable=\"yes\">Inche"
  "s (in)</item></items></object><packing><property name=\"left_attach\">1"
  "</property><property name=\"top_attach\">3</property></packing></child>"
  "<child><object class=\"GtkComboBoxText\" id=\"combo_unit_altitude\"><pr"
  "operty name=\"visible\">True</property><property name=\"can_focus\">Fal"
  "se</property><items><item translatable=\"yes\">Meters (m)</item><item t"
  "ranslatable=\"yes\">Feet (ft)</item></items></object><packing><property"
  " name=\"left_attach\">1</property><property name=\"top_attach\">4</prop"
  "erty></packing></child><child><object class=\"GtkComboBoxText\" id=\"co"
  "mbo_apparent_temperature\"><property name=\"visible\">True</property><p"
  "roperty name=\"can_focus\">False</property><items><item translatable=\""
  "yes\">Windchill/Heat index</item><item translatable=\"yes\">Windchill/H"
  "umidex</item><item translatable=\"yes\">Steadman</item><item translatab"
  "le=\"yes\">Quayle-Steadman</item></items></object><packing><property na"
  "me=\"left_attach\">1</property><property name=\"top_attach\">6</propert"
  "y></packing></child></object><packing><property name=\"position\">1</pr"
  "operty></packing></child><child type=\"tab\"><object class=\"GtkLabel\""
  " id=\"label16\"><property name=\"visible\">True</property><property nam"
  "e=\"can_focus\">False</property><property name=\"label\" translatable=\""
  "yes\">_Units</property><property name=\"use_underline\">True</property>"
  "</object><packing><property name=\"position\">1</property><property nam"
  "e=\"tab_fill\">False</property></packing></child><child><object class=\""
  "GtkGrid\" id=\"grid3\"><property name=\"visible\">True</property><prope"
  "rty name=\"can_focus\">False</property><property name=\"border_width\">"
  "12</property><property name=\"row_spacing\">6</property><property name="
  "\"column_spacing\">12</property><child><object class=\"GtkLabel\" id=\""
  "label17\"><property name=\"visible\">True</property><property name=\"ca"
  "n_focus\">False</property><property name=\"tooltip_text\" translatable="
  "\"yes\">Available icon themes are listed here. You can add icon themes "
  "to $HOME/.config/xfce4/weather/icons (or the equivalent directory on yo"
  "ur system). Information about how to create or use icon themes can be f"
  "ound in the README file. New icon themes will be detected every time yo"
  "u open this config dialog.</property><property name=\"label\" translata"
  "ble=\"yes\">_Icon theme:</property><property name=\"use_underline\">Tru"
  "e</property><property name=\"mnemonic_widget\">combo_icon_theme</proper"
  "ty><property name=\"xalign\">0</property></object><packing><property na"
  "me=\"left_attach\">0</property><property name=\"top_attach\">0</propert"
  "y></packing></child><child><object class=\"GtkBox\" id=\"box5\"><proper"
  "ty name=\"visible\">True</property><property name=\"can_focus\">False</"
  "property><property name=\"hexpand\">True</property><property name=\"spa"
  "cing\">6</property><child><object class=\"GtkComboBoxText\" id=\"combo_"
  "icon_theme\"><property name=\"visible\">True</property><property name=\""
  "can_focus\">False</property></object><packing><property name=\"expand\""
  ">True</property><property name=\"fill\">True</property><property name=\""
  "position\">0</property></packing></child><child><object class=\"GtkButt"
  "on\" id=\"button_icons_dir\"><property name=\"visible\">True</property>"
  "<property name=\"can_focus\">True</property><property name=\"receives_d"
  "efault\">True</property><property name=\"tooltip_text\" translatable=\""
  "yes\">Open the user icon themes directory in your file manager, creatin"
  "g it if necessary.</property><property name=\"image\">image2</property>"
  "</object><packing><property name=\"expand\">False</property><property n"
  "ame=\"fill\">True</property><property name=\"position\">1</property></p"
  "acking></child></object><packing><property name=\"left_attach\">1</prop"
  "erty><property name=\"top_attach\">0</property></packing></child><child"
  "><object class=\"GtkCheckButton\" id=\"check_single_row\"><property nam"
  "e=\"label\" translatable=\"yes\">Use only a single _panel row</property"
  "><property name=\"visible\">True</property><property name=\"can_focus\""
  ">True</property><property name=\"receives_default\">False</property><pr"
  "operty name=\"tooltip_text\" translatable=\"yes\">Check to always use o"
  "nly a single row on a multi-row panel and a small icon in deskbar mode."
  "</property><property name=\"use_underline\">True</property><property na"
  "me=\"draw_indicator\">True</property></object><packing><property name=\""
  "left_attach\">0</property><property name=\"top_attach\">1</property><pr"
  "operty name=\"width\">2</property></packing></child><child><object clas"
  "s=\"GtkSeparator\" id=\"separator3\"><property name=\"visible\">True</p"
  "roperty><property name=\"can_focus\">False</property><property name=\"m"
  "argin_top\">6</property><property name=\"margin_bottom\">6</property></"
  "object><packing><property name=\"left_attach\">0</property><property na"
  "me=\"top_attach\">2</property><property name=\"width\">2</property></pa"
  "cking></child><child><object class=\"GtkLabel\" id=\"label18\"><propert"
  "y name=\"visible\">True</property><property name=\"can_focus\">False</p"
  "roperty><property name=\"label\" translatable=\"yes\">_Tooltip style:</"
  "property><property name=\"use_underline\">True</property><property name"
  "=\"mnemonic_widget\">combo_tooltip_style</property><property name=\"xal"
  "ign\">0</property></object><packing><property name=\"left_attach\">0</p"
  "roperty><property name=\"top_attach\">3</property></packing></child><ch"
  "ild><object class=\"GtkComboBoxText\" id=\"combo_tooltip_style\"><prope"
  "rty name=\"visible\">True</property><property name=\"can_focus\">False<"
  "/property><property name=\"tooltip_text\" translatable=\"yes\">Choose y"
  "our preferred tooltip style. Some styles give a lot of useful data, som"
  "e are clearer but provide less data on a glance.</property><items><item"
  " translatable=\"yes\">Simple</item><item translatable=\"yes\">Verbose</"
  "item></items></object><packing><property name=\"left_attach\">1</proper"
  "ty><property name=\"top_attach\">3</property></packing></child><child><"
  "object class=\"GtkSeparator\" id=\"label19\"><property name=\"visible\""
  ">True</property><property name=\"can_focus\">False</property><property "
  "name=\"margin_top\">6</property><property name=\"margin_bottom\">6</pro"
  "perty></object><packing><property name=\"left_attach\">0</property><pro"
  "perty name=\"top_attach\">4</property><property name=\"width\">2</prope"
  "rty></packing></child><child><object class=\"GtkLabel\" id=\"label20\">"
  "<property name=\"visible\">True</property><property name=\"can_focus\">"
  "False</property><property name=\"label\" translatable=\"yes\">_Forecast"
  " layout:</property><property name=\"use_underline\">True</property><pro"
  "perty name=\"mnemonic_widget\">combo_forecast_layout</property><propert"
  "y name=\"xalign\">0</property></object><packing><property name=\"left_a"
  "ttach\">0</property><property name=\"top_attach\">5</property></packing"
  "></child><child><object class=\"GtkComboBoxText\" id=\"combo_forecast_l"
  "ayout\"><property name=\"visible\">True</property><property name=\"can_"
  "focus\">False</property><items><item translatable=\"yes\">Days in colum"
  "ns</item><item translatable=\"yes\">Days in rows</item></items></object"
  "><packing><property name=\"left_attach\">1</property><property name=\"t"
  "op_attach\">5</property></packing></child><child><object class=\"GtkLab"
  "el\" id=\"label21\"><property name=\"visible\">True</property><property"
  " name=\"can_focus\">False</property><property name=\"label\" translatab"
  "le=\"yes\">_Number of forecast days:</property><property name=\"use_und"
  "erline\">True</property><property name=\"mnemonic_widget\">spin_forecas"
  "t_days</property><property name=\"xalign\">0</property></object><packin"
  "g><property name=\"left_attach\">0</property><property name=\"top_attac"
  "h\">6</property></packing></child><child><object class=\"GtkSpinButton\""
  " id=\"spin_forecast_days\"><property name=\"visible\">True</property><p"
  "roperty name=\"can_focus\">True</property><property name=\"halign\">sta"
  "rt</property><property name=\"adjustment\">adjustment1</property><prope"
  "rty name=\"numeric\">True</property><property name=\"value\">5</propert"
  "y></object><packing><property name=\"left_attach\">1</property><propert"
  "y name=\"top_attach\">6</property></packing></child><child><object clas"
  "s=\"GtkSeparator\" id=\"separator4\"><property name=\"visible\">True</p"
  "roperty><property name=\"can_focus\">False</property><property name=\"m"
  "argin_top\">6</property><property name=\"margin_bottom\">6</property></"
  "object><packing><property name=\"left_attach\">0</property><property na"
  "me=\"top_attach\">7</property><property name=\"width\">2</property></pa"
  "cking></child><child><object class=\"GtkCheckButton\" id=\"check_round_"
  "values\"><property name=\"label\" translatable=\"yes\">_Round values</p"
  "roperty><property name=\"visible\">True</property><property name=\"can_"
  "focus\">True</property><property name=\"receives_default\">False</prope"
  "rty><property name=\"use_underline\">True</property><property name=\"dr"
  "aw_indicator\">True</property></object><packing><property name=\"left_a"
  "ttach\">0</property><property name=\"top_attach\">8</property><property"
  " name=\"width\">2</property></packing></child></object><packing><proper"
  "ty name=\"position\">2</property></packing></child><child type=\"tab\">"
  "<object class=\"GtkLabel\" id=\"label22\"><property name=\"visible\">Tr"
  "ue</property><property name=\"can_focus\">False</property><property nam"
  "e=\"label\" translatable=\"yes\">_Appearance</property><property name=\""
  "use_underline\">True</property></object><packing><property name=\"posit"
  "ion\">2</property><property name=\"tab_fill\">False</property></packing"
  "></child><child><object class=\"GtkGrid\" id=\"grid4\"><property name=\""
  "visible\">True</property><property name=\"can_focus\">False</property><"
  "property name=\"border_width\">12</property><property name=\"row_spacin"
  "g\">6</property><property name=\"column_spacing\">12</property><child><"
  "object class=\"GtkLabel\" id=\"label24\"><property name=\"visible\">Tru"
  "e</property><property name=\"can_focus\">False</property><property name"
  "=\"margin_left\">12</property><property name=\"label\" translatable=\"y"
  "es\">Font and color:</property><property name=\"xalign\">0</property></"
  "object><packing><property name=\"left_attach\">0</property><property na"
  "me=\"top_attach\">2</property></packing></child><child><object class=\""
  "GtkSeparator\" id=\"separator5\"><property name=\"visible\">True</prope"
  "rty><property name=\"can_focus\">False</property><property name=\"margi"
  "n_top\">6</property><property name=\"margin_bottom\">6</property></obje"
  "ct><packing><property name=\"left_attach\">0</property><property name=\""
  "top_attach\">4</property><property name=\"width\">2</property></packing"
  "></child><child><object class=\"GtkGrid\" id=\"grid5\"><property name=\""
  "visible\">True</property><property name=\"can_focus\">False</property><"
  "property name=\"hexpand\">True</property><property name=\"vexpand\">Tru"
  "e</property><property name=\"row_spacing\">6</property><property name=\""
  "column_spacing\">6</property><child><object class=\"GtkScrolledWindow\""
  " id=\"scrollwindow1\"><property name=\"visible\">True</property><proper"
  "ty name=\"can_focus\">True</property><property name=\"hexpand\">True</p"
  "roperty><property name=\"vexpand\">True</property><property name=\"shad"
  "ow_type\">in</property><child><object class=\"GtkViewport\" id=\"viewpo"
  "rt1\"><property name=\"visible\">True</property><property name=\"can_fo"
  "cus\">False</property><child><object class=\"GtkTreeView\" id=\"list_da"
  "tatypes\"><property name=\"visible\">True</property><property name=\"ca"
  "n_focus\">True</property><property name=\"tooltip_text\" translatable=\""
  "yes\">These are the values that will be shown in the scrollbox. Select "
  "a single value here and click the appropriate button to remove it or mo"
  "ve it up and down in the list.</property><property name=\"margin_right\""
  ">7</property><property name=\"model\">liststore1</property><child inter"
  "nal-child=\"selection\"><object class=\"GtkTreeSelection\" id=\"selecti"
  "on1\"/></child></object></child></object></child></object><packing><pro"
  "perty name=\"left_attach\">0</property><property name=\"top_attach\">1<"
  "/property></packing></child><child><object class=\"GtkBox\" id=\"box6\""
  "><property name=\"visible\">True</property><property name=\"can_focus\""
  ">False</property><property name=\"orientation\">vertical</property><chi"
  "ld><object class=\"GtkButton\" id=\"button_up\"><property name=\"visibl"
  "e\">True</property><property name=\"can_focus\">True</property><propert"
  "y name=\"receives_default\">True</property><property name=\"tooltip_tex"
  "t\" translatable=\"yes\">Move the selected value up in the list of labe"
  "ls to display.</property><property name=\"image\">image5</property><pro"
  "perty name=\"use_underline\">True</property></object><packing><property"
  " name=\"expand\">False</property><property name=\"fill\">True</property"
  "><property name=\"position\">0</property></packing></child><child><obje"
  "ct class=\"GtkButton\" id=\"button_down\"><property name=\"visible\">Tr"
  "ue</property><property name=\"can_focus\">True</property><property name"
  "=\"receives_default\">True</property><property name=\"tooltip_text\" tr"
  "anslatable=\"yes\">Move the selected value down in the list of labels t"
  "o display.</property><property name=\"image\">image6</property><propert"
  "y name=\"use_underline\">True</property></object><packing><property nam"
  "e=\"expand\">False</property><property name=\"fill\">True</property><pr"
  "operty name=\"position\">1</property></packing></child><style><class na"
  "me=\"linked\"/></style></object><packing><property name=\"left_attach\""
  ">1</property><property name=\"top_attach\">1</property></packing></chil"
  "d><child><object class=\"GtkBox\"><property name=\"visible\">True</prop"
  "erty><property name=\"can_focus\">False</property><property name=\"spac"
  "ing\">6</property><child><object class=\"GtkComboBoxText\" id=\"options"
  "_datatypes\"><property name=\"visible\">True</property><property name=\""
  "can_focus\">False</property><property name=\"hexpand\">True</property><"
  "items><item id=\"3\" translatable=\"yes\">Temperature (T)</item><item i"
  "d=\"4\" translatable=\"yes\">Barometric pressure (P)</item><item id=\"5"
  "\" translatable=\"yes\">Wind speed (WS)</item><item id=\"6\" translatab"
  "le=\"yes\">Wind speed - Beaufort scale (WB)</item><item id=\"7\" transl"
  "atable=\"yes\">Wind direction (WD)</item><item id=\"8\" translatable=\""
  "yes\">Wind direction in degrees (WD)</item><item id=\"9\" translatable="
  "\"yes\">Humidity (H)</item><item id=\"10\" translatable=\"yes\">Dew poi"
  "nt (D)</item><item id=\"11\" translatable=\"yes\">Apparent temperature "
  "(A)</item><item id=\"12\" translatable=\"yes\">Low clouds (CL)</item><i"
  "tem id=\"13\" translatable=\"yes\">Middle clouds (CM)</item><item id=\""
  "14\" translatable=\"yes\">High clouds (CH)</item><item id=\"15\" transl"
  "atable=\"yes\">Cloudiness (C)</item><item id=\"16\" translatable=\"yes\""
  ">Fog (F)</item><item id=\"17\" translatable=\"yes\">Precipitation (R)</"
  "item></items></object><packing><property name=\"expand\">False</propert"
  "y><property name=\"fill\">True</property><property name=\"position\">0<"
  "/property></packing></child><child><object class=\"GtkButton\" id=\"but"
  "ton_add\"><property name=\"visible\">True</property><property name=\"ca"
  "n_focus\">True</property><property name=\"receives_default\">True</prop"
  "erty><property name=\"tooltip_text\" translatable=\"yes\">Add the selec"
  "ted value to the labels that should be displayed in the scrollbox.</pro"
  "perty><property name=\"image\">image3</property><property name=\"use_un"
  "derline\">True</property></object><packing><property name=\"expand\">Fa"
  "lse</property><property name=\"fill\">True</property><property name=\"p"
  "osition\">1</property></packing></child><child><object class=\"GtkButto"
  "n\" id=\"button_del\"><property name=\"visible\">True</property><proper"
  "ty name=\"can_focus\">True</property><property name=\"receives_default\""
  ">True</property><property name=\"tooltip_text\" translatable=\"yes\">Se"
  "lect a value in the list of labels to display and click this button to "
  "remove it.</property><property name=\"image\">image4</property><propert"
  "y name=\"use_underline\">True</property></object><packing><property nam"
  "e=\"expand\">False</property><property name=\"fill\">True</property><pr"
  "operty name=\"position\">2</property></packing></child></object><packin"
  "g><property name=\"left_attach\">0</property><property name=\"top_attac"
  "h\">0</property><property name=\"width\">2</property></packing></child>"
  "</object><packing><property name=\"left_attach\">0</property><property "
  "name=\"top_attach\">5</property><property name=\"width\">2</property></"
  "packing></child><child><object class=\"GtkLabel\" id=\"label23\"><prope"
  "rty name=\"visible\">True</property><property name=\"can_focus\">False<"
  "/property><property name=\"margin_left\">12</property><property name=\""
  "hexpand\">True</property><property name=\"label\" translatable=\"yes\">"
  "Li_nes:</property><property name=\"use_underline\">True</property><prop"
  "erty name=\"xalign\">0</property></object><packing><property name=\"lef"
  "t_attach\">0</property><property name=\"top_attach\">1</property></pack"
  "ing></child><child><object class=\"GtkSpinButton\" id=\"spin_scrollbox_"
  "lines\"><property name=\"visible\">True</property><property name=\"can_"
  "focus\">True</property><property name=\"tooltip_text\" translatable=\"y"
  "es\">Decide how many values should be shown at once in the scrollbox. Y"
  "ou can choose a smaller font or enlarge the panel to make more lines fi"
  "t.</property><property name=\"halign\">end</property><property name=\"a"
  "djustment\">adjustment2</property><property name=\"numeric\">True</prop"
  "erty><property name=\"value\">1</property></object><packing><property n"
  "ame=\"left_attach\">1</property><property name=\"top_attach\">1</proper"
  "ty></packing></child><child><object class=\"GtkSwitch\" id=\"check_scro"
  "llbox_show\"><property name=\"visible\">True</property><property name=\""
  "can_focus\">True</property><property name=\"tooltip_text\" translatable"
  "=\"yes\">Hide the scrollbox to save valuable space on the panel. Most i"
  "nteresting information is also provided in the tooltip - provided you c"
  "hoose an appropriate tooltip style - that is shown when hovering over t"
  "he icon.</property><property name=\"halign\">end</property><property na"
  "me=\"valign\">center</property></object><packing><property name=\"left_"
  "attach\">1</property><property name=\"top_attach\">0</property></packin"
  "g></child><child><object class=\"GtkLabel\"><property name=\"visible\">"
  "True</property><property name=\"can_focus\">False</property><property n"
  "ame=\"label\" translatable=\"yes\">&lt;b&gt;Show scrollbox&lt;/b&gt;</p"
  "roperty><property name=\"use_markup\">True</property><property name=\"x"
  "align\">0</property></object><packing><property name=\"left_attach\">0<"
  "/property><property name=\"top_attach\">0</property></packing></child><"
  "child><object class=\"GtkBox\"><property name=\"visible\">True</propert"
  "y><property name=\"can_focus\">False</property><property name=\"halign\""
  ">end</property><property name=\"spacing\">6</property><child><object cl"
  "ass=\"GtkButton\" id=\"button_scrollbox_font\"><property name=\"label\""
  " translatable=\"yes\">Select _font</property><property name=\"visible\""
  ">True</property><property name=\"can_focus\">True</property><property n"
  "ame=\"receives_default\">True</property><property name=\"tooltip_text\""
  " translatable=\"yes\">Choose a font you like and set it to a smaller or"
  " larger size. Middle-click on the button to unset the font and use your"
  " theme\'s default.</property><property name=\"halign\">center</property"
  "><property name=\"valign\">center</property><property name=\"use_underl"
  "ine\">True</property></object><packing><property name=\"expand\">False<"
  "/property><property name=\"fill\">True</property><property name=\"posit"
  "ion\">0</property></packing></child><child><object class=\"GtkColorButt"
  "on\" id=\"button_scrollbox_color\"><property name=\"visible\">True</pro"
  "perty><property name=\"can_focus\">True</property><property name=\"rece"
  "ives_default\">True</property><property name=\"tooltip_text\" translata"
  "ble=\"yes\">There may be problems with some themes that cause the scrol"
  "lbox text to be hardly readable. If this is the case or you simply want"
  " it to appear in another color, then you can change it using this butto"
  "n. Middle-click on the button to unset the scrollbox text color.</prope"
  "rty><property name=\"halign\">end</property><property name=\"valign\">c"
  "enter</property></object><packing><property name=\"expand\">False</prop"
  "erty><property name=\"fill\">True</property><property name=\"position\""
  ">1</property></packing></child></object><packing><property name=\"left_"
  "attach\">1</property><property name=\"top_attach\">2</property></packin"
  "g></child><child><object class=\"GtkLabel\"><property name=\"visible\">"
  "True</property><property name=\"can_focus\">False</property><property n"
  "ame=\"margin_left\">12</property><property name=\"label\" translatable="
  "\"yes\">Animate transitions between labels</property><property name=\"x"
  "align\">0</property></object><packing><property name=\"left_attach\">0<"
  "/property><property name=\"top_attach\">3</property></packing></child><"
  "child><object class=\"GtkSwitch\" id=\"check_scrollbox_animate\"><prope"
  "rty name=\"visible\">True</property><property name=\"can_focus\">True</"
  "property><property name=\"tooltip_text\" translatable=\"yes\">Scroll th"
  "e current displayed value(s) out and the new value(s) in instead of sim"
  "ply changing them. Uncheck this option if you find the animation too di"
  "stracting.</property><property name=\"halign\">end</property><property "
  "name=\"valign\">center</property></object><packing><property name=\"lef"
  "t_attach\">1</property><property name=\"top_attach\">3</property></pack"
  "ing></child></object><packing><property name=\"position\">3</property><"
  "/packing></child><child type=\"tab\"><object class=\"GtkLabel\" id=\"la"
  "bel25\"><property name=\"visible\">True</property><property name=\"can_"
  "focus\">False</property><property name=\"label\" translatable=\"yes\">_"
  "Scrollbox</property><property name=\"use_underline\">True</property></o"
  "bject><packing><property name=\"position\">3</property><property name=\""
  "tab_fill\">False</property></packing></child></object><packing><propert"
  "y name=\"expand\">True</property><property name=\"fill\">True</property"
  "><property name=\"position\">1</property></packing></child></object></c"
  "hild><action-widgets><action-widget response=\"-11\">help_button</actio"
  "n-widget><action-widget response=\"-7\">close_button</action-widget></a"
  "ction-widgets></object></interface>"
};

static const unsigned weather_config_ui_length = 42403u;

